function TF_escape( str ) {

    if ( ! str ) return "";
    str = "" + str;

    str = str.replace( /\|/g, '&_or_;' );
    str = str.replace( /\#/g, '&_hash_;' );
    return str;
}
function TF_unescape( str ) {

    if ( ! str ) return "";
    str = "" + str;  

    str = str.replace( /\&_or_\;/g, '|' );
    str = str.replace( /\&_hash_\;/g, '#' );
    return str;
}
function TF_strToTuple( str ) {

    if ( ! str ) {
        return new Array();
    }

    var raw_tuple = str.split( "|" );
    var out_tuple = new Array();
    
    for ( var i = 0; i < raw_tuple.length; i++ ) {
        out_tuple.push( TF_unescape( raw_tuple[i] ));
    }
    
    return out_tuple;
}
function TF_tupleToStr( in_tuple ) {

  var esc_tuple = new Array();

  for ( var i = 0; i < in_tuple.length; i++ ) {
      esc_tuple.push( TF_escape( in_tuple[i] ));
  }

  return esc_tuple.join( "|" );

}
const PFMC_BEFORE = 'before';
const PFMC_AFTER = 'after';
const PFMC_IGNORE = 'ignore';
const PFMC_WILDCARD = "*";
const PRONTOFOXMM_CONTRACTID = '@pronto.com/ProntoFoxMM;1';
const PRONTOFOXMM_CID = Components.ID('{116a8ba9-80ad-4116-83fb-9643baa3d9e4}');
const PRONTOFOXMM_IID = Components.interfaces.nsIProntoFoxMM;
const PF_GUID = "{6aec4bf7-c16a-4e5c-a65a-114a57157969}";
const SITE_MODEL_DIR = "site-model";
const PFMMX_GLOBAL = "GLOBAL";
const PFMMX_DELETE = "DELETE";
const PFMMX_DEFAULT_GLOBAL_URL = PFMC_BEFORE;
const PFMMX_DEFAULT_GLOBAL_TL = PFMC_BEFORE;
const PFMMX_SITE_MODEL_VERSION = "currentLiteModelVersion";
const PFMMX_SITE_MODEL_ID_LABEL = "ProntoSiteModelID";
var PFMMX_SITE_MODEL_ID_RE 
        = new RegExp( "^\\s*\\#\\s*" + PFMMX_SITE_MODEL_ID_LABEL +
                      "\\s*=\\s*(\\S+)", "i" );
const PFMMX_SM_TYPE_GLOBAL_URL = 'GlobalURLRegex';
const PFMMX_SM_TYPE_GLOBAL_TL = 'GlobalTitleLocator';
const PFMMX_SM_TYPE_URL = 'URLRegex';
const PFMMX_SM_TYPE_TL = 'TitleLocator';
const PFMMX_TL_TYPE_TAG = 'HtmlTag';
const PFMMX_TL_TYPE_ATTR = 'TagAttr';
const PFMMX_TL_TYPE_VALUE = 'AttrValue';
const PFMMX_TL_TYPE_MATCH = 'MatchRE';
const PFMMX_TL_TYPE_EXTRACT = 'ExtractRE';
const PFMMX_PREF_BRANCH_LABEL = "pronto.mm.";

var PFMMX_prefBranch = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService).getBranch( PFMMX_PREF_BRANCH_LABEL );
var PFMMX_SiteModelList = new Array();
var PFMMX_GlobalSiteModel = null;

var PFMMX_DEBUG_ENABLED = false;
var PFMMX_PARSE_DEBUG = false;
var PFMMX_UPDATE_DEBUG = false;
var PFMMX_XMLHTTP_DEBUG = false;

var _consoleService 
        = Components.classes["@mozilla.org/consoleservice;1"].getService(Components.interfaces.nsIConsoleService);

function PFMMX_DEBUG(msg) {
    if ( ! PFMMX_DEBUG_ENABLED ) {
        return;
    }
    _consoleService.logStringMessage("PFMMX: " + msg);
}
function _error( msg ) {
    _consoleService.logStringMessage("PFMMX ERROR: " + msg);
}
function _warning( msg ) {
    _consoleService.logStringMessage("PFMMX WARN: " + msg);
}
function nsProntoFoxMM() { 

    PFMMX_DEBUG_ENABLED = PFMMX_prefBranch.getBoolPref("debug.xpcom.main") ;
    PFMMX_PARSE_DEBUG = PFMMX_prefBranch.getBoolPref("debug.xpcom.parse") ;
    PFMMX_UPDATE_DEBUG = PFMMX_prefBranch.getBoolPref("debug.xpcom.update") ;
    PFMMX_XMLHTTP_DEBUG = PFMMX_prefBranch.getBoolPref("debug.xpcom.xmlhttp") ;

    PFMMX_ReadSiteModels();

    _prefObserver = new PFMMX_prefObserver();
    
    _initTimer();
    
}

nsProntoFoxMM.prototype = {
    getSiteModel: function( site, count ) {
        
        var model_array;
        
        if ( PFMMX_SiteModelList[site] ) {

            if ( PFMMX_DEBUG_ENABLED )
                PFMMX_DEBUG( PFMMX_SiteModelList[site].toString() );

            model_array = PFMMX_SiteModelList[site].getAsArray();
        }
        else {
            model_array = new Array();
        }
        
        if ( PFMMX_DEBUG_ENABLED )
             PFMMX_DEBUG( "Returning: " + model_array.join( "\n") );

        count.value = model_array.length;
        return model_array;

    },
    getGlobalSiteModel: function( count ) {

        if ( PFMMX_GlobalSiteModel == null ) {
            _error( "No global site model to return." )
            return new Array();
        }
        PFMMX_GlobalSiteModel.globalURL = PFMC_IGNORE;
        PFMMX_GlobalSiteModel.globalTL = PFMC_IGNORE;

        var model_array = PFMMX_GlobalSiteModel.getAsArray();

        if ( PFMMX_DEBUG_ENABLED )
             PFMMX_DEBUG( "Returning: " + model_array.join( "\n") );

        count.value = model_array.length;
        return model_array;

    },
    QueryInterface: function(iid) {
        if (!iid.equals(Components.interfaces.nsISupports) &&
            !iid.equals(PRONTOFOX_IID))
        throw Components.results.NS_ERROR_NO_INTERFACE;
        return this;
    }


};

var nsProntoFoxMMModule = {

    registerSelf: function(compMgr, fileSpec, location, type) {
        compMgr = compMgr.QueryInterface(Components.interfaces.nsIComponentRegistrar);
        compMgr.registerFactoryLocation(PRONTOFOXMM_CID,
                                        "ProntoFoxMM XPCOM",
                                        PRONTOFOXMM_CONTRACTID,
                                        fileSpec,
                                        location,
                                        type);
    },
    
    getClassObject: function(compMgr, cid, iid) {
        if (!cid.equals(PRONTOFOXMM_CID))
            throw Components.results.NS_ERROR_NO_INTERFACE;
        if (!iid.equals(Components.interfaces.nsIFactory))
            throw Components.results.NS_ERROR_NOT_IMPLEMENTED;
        return nsProntoFoxMMFactory;   
    },
    canUnload: function(compMgr) { return true; }
};

var nsProntoFoxMMFactory = {
    createInstance: function(outer, iid) {
        if (outer != null)
            throw Components.results.NS_ERROR_NO_AGGREGATION;
        if (!iid.equals(PRONTOFOXMM_IID) &&
            !iid.equals(Components.interfaces.nsISupports))
            throw Components.results.NS_ERROR_INVALID_ARG;
        return new nsProntoFoxMM();
    }
};

function NSGetModule(comMgr, fileSpec) { return nsProntoFoxMMModule; }
function PFMMX_regexPerlToJS( re ) {
    re = re.replace( /\(\?i\)/, '' );

    return re;
}
function PFMMX_SiteModel( site_name ) {
    this._init( site_name );
}

PFMMX_SiteModel.prototype = {
    _init: function ( site_name ) {
        this.site_name = site_name;
        
    },
    addItem: function( fields ) {
        if ( fields.length < 1 ) return;

        var type = fields.shift();

        switch ( type ) {
        case PFMMX_SM_TYPE_GLOBAL_URL:
            this.globalURL = fields[0];
            break;

        case PFMMX_SM_TYPE_GLOBAL_TL:
            this.globalTL = fields[0];
            break;
            
        case PFMMX_SM_TYPE_URL:
            var re = PFMMX_regexPerlToJS( fields[0] );
            if ( re == "" ) {
                _warning( "Empty RE for URLRegex not allowed" );
                return;
            }
            if ( ! this.urlList ) {
                this.urlList = new Array();
            }
            this.urlList.push( re );
            break;
            
        case PFMMX_SM_TYPE_TL:
            if (( fields.length < 1 ) 
                || ( fields[0] == "" ) ) {
                fields[0] = PFMC_WILDCARD;
            }
            else {
                fields[0] = fields[0].toLowerCase();
            }
            if (( fields.length < 2 ) 
                || ( fields[1] == "" ) ) {
                fields[1] = PFMC_WILDCARD;
            }
            if (( fields.length < 3 ) 
                || ( fields[2] == "" ) ) {
                fields[2] = PFMC_WILDCARD;
            }
            if (( fields.length < 4 ) 
                || ( fields[3] == "" ) ) {
                fields[3] = ".";
            }
            else {
                var re = PFMMX_regexPerlToJS( fields[3] );
                if ( re == "" ) {
                    _warning( "Empty RE for MatchRegex not allowed" );
                    return;
                }
                fields[3] = PFMMX_regexPerlToJS( re )
            }
            if (( fields.length < 5 ) 
                || ( fields[4] == "" ) ) {
                fields[4] = "^(.*)$";
            }
            else {
                var re = PFMMX_regexPerlToJS( fields[4] );
                if ( re == "" ) {
                    _warning( "Empty RE for ExtractRegex not allowed" );
                    return;
                }
                fields[4] = PFMMX_regexPerlToJS( re )
            }
            
            if ( ( fields[0] == PFMC_WILDCARD ) 
                 && ( fields[1] == PFMC_WILDCARD ) 
                 && ( fields[2] == PFMC_WILDCARD )) {
                _warning( "Invalid locator discarded" );
                return;
            }
            
            if ( ! this.tlList ) {
                this.tlList = new Array();
            }

            this.tlList.push( TF_tupleToStr( fields ) );
            break;
        default:
            _warning( "setItem: Unrecognized title-locator type: " + type );
            break;
        }
       
    },

    toString: function () {
        var str = "SiteModel: " + this.site_name + "\n";

        if ( this.globalURL ) {
            str += "  GlobalURLRegex: " + this.globalURL + "\n";
        }
        else {
            str += "  GlobalURLRegex: " + PFMMX_DEFAULT_GLOBAL_URL + "\n";
        }

        if ( this.globalTL ) {
            str += "  GlobalTitleLocators: " + this.globalTL + "\n";
        }
        else {
            str += "  GlobalTitleLocators: " + PFMMX_DEFAULT_GLOBAL_TL + "\n";
        }

        str += "  URLRegex:\n";
        if ( this.urlList ) {
            for ( var i = 0; i < this.urlList.length; i++ ) {
                str += "    " + this.urlList[i] + "\n";
            }
        }
        else {
            str += "    NONE\n";
        }

        str += "  TitleLocators:\n";
        if ( this.tlList ) {
            for ( var i = 0; i < this.tlList.length; i++ ) {
                str += "    " + this.tlList[i] + "\n";
            }
        }
        else {
            str += "    NONE\n";
        }
        
        return str;
    },
    getAsArray: function () {

        var model_array = new Array();

        model_array.push( this.site_name );

        if ( this.globalURL ) {
            model_array.push( this.globalURL );
        }
        else {
            model_array.push( PFMMX_DEFAULT_GLOBAL_URL );
        }

        if ( this.globalTL ) {
            model_array.push( this.globalTL );
        }
        else {
            model_array.push( PFMMX_DEFAULT_GLOBAL_TL );
        }
        
        if ( this.urlList ) {
            model_array.push( TF_tupleToStr( this.urlList ));
        }
        else {
            model_array.push( "" );
        }

        if ( this.tlList ) {
            model_array = model_array.concat( this.tlList );
        }

        return model_array;
    }
}

function PFMMX_processSiteModelLine( sm_spec_str ) {

    if ( /^\s*$/.test( sm_spec_str ) ) return;

    if ( PFMMX_PARSE_DEBUG ) 
        PFMMX_DEBUG( "Processing line: " + sm_spec_str );

    var tuple = TF_strToTuple( sm_spec_str );
    var site = tuple.shift();
    if (( tuple.length == 1 ) && ( tuple[0] == PFMMX_DELETE )) {

        if ( PFMMX_PARSE_DEBUG ) 
            PFMMX_DEBUG( "Deleting site: " + site );

        if ( site == PFMMX_GLOBAL ) {
            PFMMX_GlobalSiteModel = null;
        }
        else {
            PFMMX_SiteModelList[site] = null;
        }
        return;
    } // special case of 'DELETE'
    
    var sm;
    if ( site == PFMMX_GLOBAL ) {
        sm = PFMMX_GlobalSiteModel;
    }
    else {
        if ( PFMMX_SiteModelList[site] == null ) {
            if ( PFMMX_PARSE_DEBUG ) 
                PFMMX_DEBUG( "Creating model for: : " + site );
            PFMMX_SiteModelList[site] = new PFMMX_SiteModel( site );;
        }
        sm = PFMMX_SiteModelList[site];
    }
    
    if ( sm == null ) {
        _error( "Site model null tying to add line" );
        return;
    }

    if ( tuple.length > 0 ) {
        if ( PFMMX_PARSE_DEBUG ) 
            PFMMX_DEBUG( "Adding Item: " + tuple );
        sm.addItem( tuple );
    }
    
}
const PFMMX_DEFAULT_SM_FILE = "pf-default.sm";
const PFMMX_ADDITIONAL_SM_FILE = "pf-additional.sm";
const PFMMX_CURRENT_SM_FILE = "pf-current.sm";
const PFMMX_LOCAL_SM_FILE = PFMMX_prefBranch.getCharPref("model_file.local");
function PFMMX_ReadSiteModels() {
    PFMMX_GlobalSiteModel = new PFMMX_SiteModel( PFMMX_GLOBAL );
    PFMMX_SiteModelList = new Array();

    var file_read = false;
    var file = PFMMX_getExtensionFile( SITE_MODEL_DIR, 
                                      PFMMX_ADDITIONAL_SM_FILE );
    
    if( file.exists() ) {
        if ( PFMMX_DEBUG_ENABLED )
            PFMMX_DEBUG( "Reading from 'additional' site model file." );
        PF_readSiteModelsFromFile( file );
    }
    else {
        PFMMX_DEBUG( "No 'additonal' site model file." );
    }
    file = PFMMX_getExtensionFile( SITE_MODEL_DIR, 
                                  PFMMX_CURRENT_SM_FILE );
    if( file.exists() ) {
        if ( PFMMX_DEBUG_ENABLED )
            PFMMX_DEBUG( "Reading from 'current' site model file." );
        file_read = PF_readSiteModelsFromFile( file );
    }
    
    else {
        if ( PFMMX_DEBUG_ENABLED )
            PFMMX_DEBUG( "No 'current' site model file." );
    }
    if ( ! file_read ) {

        file = PFMMX_getExtensionFile( SITE_MODEL_DIR, 
                                      PFMMX_DEFAULT_SM_FILE );
        if( file.exists() ) {
            if ( PFMMX_DEBUG_ENABLED )
                PFMMX_DEBUG( "Reading from 'default' site model file." );
            file_read = PF_readSiteModelsFromFile( file );
        }
        
        else {
            if ( PFMMX_DEBUG_ENABLED )
                PFMMX_DEBUG( "No 'default' site model file." );
        }
    }
    file = PFMMX_getProfileFile( PFMMX_LOCAL_SM_FILE );
    
    if( file.exists() ) {
        if ( PFMMX_DEBUG_ENABLED )
            PFMMX_DEBUG( "Reading from 'local' site model file." );
        PF_readSiteModelsFromFile( file );
    }
    else {
        PFMMX_DEBUG( "No 'local' site model file." );
    }

    if ( PFMMX_DEBUG_ENABLED ) {
        var str = PFMMX_GlobalSiteModel.toString();
        for (var site in PFMMX_SiteModelList) {
            if ( PFMMX_SiteModelList[site] == null ) continue;
            str += PFMMX_SiteModelList[site].toString();
        }
        PFMMX_DEBUG( "SiteModels:\n" + str );
    }    
}
function PFMMX_getProfileFile( filename ) {

    var file = Components.classes["@mozilla.org/file/directory_service;1"].getService(Components.interfaces.nsIProperties).get("ProfD", Components.interfaces.nsIFile);

    file.append( filename );
    
    if ( PFMMX_DEBUG_ENABLED )
        PFMMX_DEBUG( "Profile file path: " + file.path );

    return file;
}
function PFMMX_getExtensionFile( top_dir, filename ) {

  var file = Components.classes["@mozilla.org/file/directory_service;1"]
                     .getService(Components.interfaces.nsIProperties)
                     .get("ProfD", Components.interfaces.nsIFile);

  file.append( "extensions" );
  file.append( PF_GUID );
  file.append( top_dir );
  file.append( filename );
  
  if ( PFMMX_DEBUG_ENABLED )
      PFMMX_DEBUG( "Extension file path: " + file.path );
  
  return file;

}
function PF_readSiteModelsFromFile( file ) {
    var istream = Components.classes["@mozilla.org/network/file-input-stream;1"].createInstance(Components.interfaces.nsIFileInputStream);
    istream.init(file, 0x01, 0444, 0);
    istream.QueryInterface(Components.interfaces.nsILineInputStream);

    var model_id = null;
    var line = {}, hasmore;
    do {
        hasmore = istream.readLine(line);
        
        var mr = line.value.match( PFMMX_SITE_MODEL_ID_RE );
        if ( mr ) {
            model_id = mr[1];
            if ( PFMMX_DEBUG_ENABLED ) 
                PFMMX_DEBUG( "Model reading found ID: " + model_id );
        }
        if ( /^\s*$/.test( line.value ) ) continue;
        if ( /^\s*\#/.test( line.value ) ) continue;
    
        PFMMX_processSiteModelLine( line.value );

    } while(hasmore);
    
    istream.close();
    if ( model_id )
        PFMMX_prefBranch.setCharPref( PFMMX_SITE_MODEL_VERSION, 
                                     model_id );
    
    return true;

}
function  PF_writeSiteModels( new_model_id, sm_file_content ) {

    if ( PFMMX_DEBUG_ENABLED )
        PFMMX_DEBUG( "New Model ID to write: " + new_model_id );

    try {
        var file = PFMMX_getExtensionFile( SITE_MODEL_DIR, 
                                          PFMMX_CURRENT_SM_FILE );
        var foStream = Components.classes["@mozilla.org/network/file-output-stream;1"].createInstance(Components.interfaces.nsIFileOutputStream);
        foStream.init(file, 0x02 | 0x08 | 0x20, 0664, 0); 
        var converter = Components.classes["@mozilla.org/intl/scriptableunicodeconverter"].createInstance(Components.interfaces.nsIScriptableUnicodeConverter);
        converter.charset = "UTF-8";
        
        var chunk = converter.ConvertFromUnicode( sm_file_content );
        foStream.write(chunk, chunk.length);
        
        var fin = converter.Finish();
        if (fin.length > 0)
            foStream.write(fin, fin.length);
        foStream.close();
        
    }
    catch (failed) {
        _error( "Problem writing site models to file." );
        return false;
    }
    PFMMX_prefBranch.setCharPref( PFMMX_SITE_MODEL_VERSION, 
                                 new_model_id );
    
    return true;
}

var PFMMX_UPDATE_INTERVAL_MIN 
        = PFMMX_prefBranch.getIntPref("update.interval.min") ;
var PFMMX_UPDATE_INITIAL_MIN = 10;
var PFMMX_TIMER_INTERVAL_MS = 60 * 1000;
if ( PFMMX_UPDATE_DEBUG ) {
    PFMMX_TIMER_INTERVAL_MS = 1000;
    PFMMX_DEBUG( "Model update interval timer overridden to be 1 second" );
}

var _timer = Components.classes["@mozilla.org/timer;1"].getService(Components.interfaces.nsITimer);
var PFMMX_nonUpdateQueryIntervalCount 
        = PFMMX_UPDATE_INTERVAL_MIN - PFMMX_UPDATE_INITIAL_MIN;
var _timerObserver = {
    notify: function() {
        if ( PFMMX_DEBUG_ENABLED )
             PFMMX_DEBUG( "Interval timer triggered." );

        if ( PFMMX_UPDATE_INTERVAL_MIN == 0 ) {
            if ( PFMMX_DEBUG_ENABLED )
                PFMMX_DEBUG( "Model update querying is disabled." );
            return;
        }

        PFMMX_nonUpdateQueryIntervalCount += 1;

        if ( PFMMX_nonUpdateQueryIntervalCount < PFMMX_UPDATE_INTERVAL_MIN ) {
            if ( PFMMX_DEBUG_ENABLED )
                PFMMX_DEBUG( "Not time to query for a site model update." );
            return;
        }

        PF_sendSiteModelRequest();

        PFMMX_nonUpdateQueryIntervalCount = 0;
    }
}
function _initTimer() {
    _timer.initWithCallback
            ( _timerObserver,
              PFMMX_TIMER_INTERVAL_MS,
              Components.interfaces.nsITimer.TYPE_REPEATING_SLACK );

}
var PFMMX_curXMLHTTPRequest = false;
function PF_sendSiteModelRequest(  ) {


    if ( PFMMX_XMLHTTP_DEBUG ) 
        PFMMX_DEBUG( "Query for new site model data" );

    PFMMX_curXMLHTTPRequest_request = false;
    try {
        PFMMX_curXMLHTTPRequest = Components.classes["@mozilla.org/xmlextras/xmlhttprequest;1"].createInstance();
    } catch (failed) {
        PFMMX_curXMLHTTPRequest = false;
    }
    
    if ( ! PFMMX_curXMLHTTPRequest ) {
        _error("Error initializing XMLHttpRequest!");
        return;
    }
    
    
    var model_id = PFMMX_prefBranch.getCharPref( PFMMX_SITE_MODEL_VERSION );
    var url = "http://" 
            + PFMMX_prefBranch.getCharPref( "update.host" )
            + PFMMX_prefBranch.getCharPref( "update.prefix" )
            + "?" + PFMMX_SITE_MODEL_VERSION + "=" + model_id;
    
    if ( PFMMX_XMLHTTP_DEBUG )
        PFMMX_DEBUG( "XMLHTTP URL: " + url );

    PFMMX_curXMLHTTPRequest.open('GET', url, true);

    PFMMX_curXMLHTTPRequest.setRequestHeader( 'User-Agent', 
                                             'ProntoFoxMM'); 

    PFMMX_curXMLHTTPRequest.onreadystatechange = PF_handleSiteModelReply;

    if ( PFMMX_XMLHTTP_DEBUG ) 
        PFMMX_DEBUG( "Sending site model query request" );

    PFMMX_curXMLHTTPRequest.send( null );
    

}
function PF_handleSiteModelReply() {

    if ( ! PFMMX_curXMLHTTPRequest ) {
        if ( PFMMX_XMLHTTP_DEBUG ) 
            PFMMX_DEBUG( "Request object doesn't exist" );
        return;
    }

    if ( PFMMX_curXMLHTTPRequest.readyState != 4 ) {
        return;
    }

    if ( PFMMX_XMLHTTP_DEBUG ) 
        PFMMX_DEBUG( "Handling site model query reply" );

    try {
        if ( PFMMX_curXMLHTTPRequest.status != 200) {
            if ( PFMMX_XMLHTTP_DEBUG ) 
                PFMMX_DEBUG( "Problem. Pronto Server returned status " 
                            + PFMMX_curXMLHTTPRequest.status 
                            + " for  XMLHTTP request " );
            return;
        }
    }
    catch (failed) {
        if ( PFMMX_XMLHTTP_DEBUG ) 
            PFMMX_DEBUG( "XMLHTTP Network error."  );
        return;
    }
    
    var reply_text = PFMMX_curXMLHTTPRequest.responseText;
    
    if ( PFMMX_XMLHTTP_DEBUG ) 
        PFMMX_DEBUG( "XMLHTTP Reply Content:\n" + reply_text );
    var mr = reply_text.match( PFMMX_SITE_MODEL_ID_RE );
    if ( ! mr ) {
        if ( PFMMX_XMLHTTP_DEBUG ) 
            PFMMX_DEBUG( "Model query did not return a model." );
        return;
    }

    if ( PFMMX_DEBUG_ENABLED )
        PFMMX_DEBUG( "ID Regex results: " + mr );
    if ( PF_writeSiteModels( mr[1], reply_text ) ) {
        PFMMX_ReadSiteModels();
    }
    
}
var _prefObserver;
function PFMMX_prefObserver() {
    this.register();
}

PFMMX_prefObserver.prototype = {

  register: function() {
      var prefService 
      = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService);
      this._branch = prefService.getBranch( PFMMX_PREF_BRANCH_LABEL );
      this._branch.QueryInterface(Components.interfaces.nsIPrefBranchInternal);
      this._branch.addObserver("", this, false);
   },

  unregister: function() {
      if(!this._branch) return;
      this._branch.removeObserver("", this);
  },

  observe: function(aSubject, aTopic, aData) {
      if ( aTopic != "nsPref:changed" ) return;
      if ( ! Components ) return;
      switch (aData) {
      case "debug.xpcom.main":
          PFMMX_DEBUG_ENABLED = this._branch.getBoolPref("debug.xpcom.main") ;
          break;
      case "debug.xpcom.xmlhttp":
          PFMMX_XMLHTTP_DEBUG = this._branch.getBoolPref("debug.xpcom.xmlhttp") ;
          break;
      case "debug.xpcom.parse":
          PFMMX_PARSE_DEBUG = this._branch.getBoolPref("debug.xpcom.parse") ;
          break;
      case "debug.xpcom.update":
          PFMMX_UPDATE_DEBUG = this._branch.getBoolPref("debug.xpcom.update") ;
          break;
      case "update.interval.min":
          PFMMX_UPDATE_INTERVAL_MIN = this._branch.getIntPref("update.interval.min") ;
          break;
      case "model_file.main":
          PFMMX_CURRENT_SM_FILE = this._branch.getCharPref("model_file.main") ;
          break;
      case "model_file.local":
          PFMMX_LOCAL_SM_FILE = this._branch.getCharPref("model_file.local") ;
          break;
      }
  }
}
