
const PRONTOPLATFORM_CONTRACTID = '@pronto.com/ProntoPlatform;1';
const PRONTOPLATFORM_CID = Components.ID('{d4bd6594-632d-4290-ba26-854378a72bc9}');
const PRONTOPLATFORM_IID = Components.interfaces.nsIProntoPlatform;
var _prefBranch = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService).getBranch("pronto.");
var _consoleService 
        = Components.classes["@mozilla.org/consoleservice;1"].getService(Components.interfaces.nsIConsoleService);

function PFP_DEBUG(msg) {
    if ( ! _prefBranch.getBoolPref("debug.platform") )
        return;
    _consoleService.logStringMessage("PFP_DEBUG: " + msg);
}
function _initialize() {
}
function nsProntoPlatform() { 

    _initialize();

}

nsProntoPlatform.prototype = {
    generateMachineID: function( ) {
        return PFP_generateMachineID();
        
    },
    setPlatformSpecificPrefs: function( platform, user_agent ) {
        PFP_setPlatformSpecificPrefs( platform, user_agent );
    },
    QueryInterface: function(iid) {
        if (!iid.equals(Components.interfaces.nsISupports) &&
            !iid.equals(PRONTOPLATFORM_IID))
        throw Components.results.NS_ERROR_NO_INTERFACE;
        return this;
    }


};

function PFP_generateMachineID() {

    var identifier = null;
    if ( ( ! identifier ) || ( identifier == "" )) {
        identifier = PFP_semiUniqueIdentifier();
    }

    return identifier;

}
var PFP_MACCommand = new Array();
PFP_MACCommand.push( 'winipcfg' );       // For win9x
PFP_MACCommand.push( 'ipconfig /all' );
PFP_MACCommand.push( 'ifconfig' );       // GNU/Linux, Mac OS-X, other unixes
PFP_MACCommand.push( '/sbin/ifconfig' ); // GNU/Linux, Mac OS-X, other unixes

function PFP_getMACAddress() {

    return null;

}
function PFP_semiUniqueIdentifier() {
    var date_part = new String( (new Date()).valueOf() );
    
    var rand_part = new String( Math.random() );
    rand_part = rand_part.replace( /^0\./, "" ); 
    
    return date_part + ":" + rand_part;
}
function PFP_setPlatformSpecificPrefs( platform, user_agent ) {

    if ( platform.indexOf("win32") != -1 
         || platform.indexOf("win16") != -1 ) {
        PFP_DEBUG( "Windows Platform: " + platform );
    }
    
    if ( platform.indexOf("macppc") != -1
         || platform.indexOf("mac68k") != -1 ) {
        PFP_DEBUG( "Macintosh Platform: " + platform );
        _prefBranch.setIntPref( "toast.window.extra.height", 20 );

    }
    
    if ( platform.indexOf("linux") != -1 ) {

        PFP_DEBUG( "GNU/Linux Platform: " + platform );
        _prefBranch.setIntPref( "toast.window.extra.height", 20 );

        _prefBranch.setCharPref( "toast.slide.direction", "diagonal" );
        _prefBranch.setIntPref( "toast.slide.offset", 1 );
    }

    PFP_DEBUG( "User Agent: " + user_agent );

    var re = new RegExp( "firefox.([0-9]+).([0-9]+)", "i" );
    var mr = user_agent.match( re );
    if ( mr ) {

        var ff_major = mr[1];
        var ff_minor = mr[2];
 
        PFP_DEBUG( "Firefox : major=" + ff_major + ", minor=" + ff_minor );

    }
    
  
}

var nsProntoPlatformModule = {

    registerSelf: function(compMgr, fileSpec, location, type) {
        compMgr = compMgr.QueryInterface(Components.interfaces.nsIComponentRegistrar);
        compMgr.registerFactoryLocation(PRONTOPLATFORM_CID,
                                        "ProntoFox XPCOM",
                                        PRONTOPLATFORM_CONTRACTID,
                                        fileSpec,
                                        location,
                                        type);
    },
    
    getClassObject: function(compMgr, cid, iid) {
        if (!cid.equals(PRONTOPLATFORM_CID))
            throw Components.results.NS_ERROR_NO_INTERFACE;
        if (!iid.equals(Components.interfaces.nsIFactory))
            throw Components.results.NS_ERROR_NOT_IMPLEMENTED;
        return nsProntoPlatformFactory;   
    },
    canUnload: function(compMgr) { return true; }
};

var nsProntoPlatformFactory = {
    createInstance: function(outer, iid) {
        if (outer != null)
            throw Components.results.NS_ERROR_NO_AGGREGATION;
        if (!iid.equals(PRONTOPLATFORM_IID) &&
            !iid.equals(Components.interfaces.nsISupports))
            throw Components.results.NS_ERROR_INVALID_ARG;
        return new nsProntoPlatform();
    }
};

function NSGetModule(comMgr, fileSpec) { return nsProntoPlatformModule; }
