

const PRONTOFOX_CONTRACTID = '@pronto.com/ProntoFox;1';
const PRONTOFOX_CID = Components.ID('{0cdf49fe-4e00-4c95-b40b-e90534855321}');
const PRONTOFOX_IID = Components.interfaces.nsIProntoFox;

var MAX_TOAST_HISTORY_LENGTH = 20;

var _toastHistory = new Array();
var _toastTitleHistory = new Array();

var _sessionCounter = 0;

var _has_alert_service = false;
var PFX_PrefBranch = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService).getBranch("pronto.");

var PFX_DEBUG_ENABLED = false;

var PFX_ConsoleService 
        = Components.classes["@mozilla.org/consoleservice;1"].getService(Components.interfaces.nsIConsoleService);

var PFX_DEBUG_SNOOZE = PFX_PrefBranch.getBoolPref('debug.snooze');

function PFX_DEBUG(msg) {
    if ( ! PFX_DEBUG_ENABLED ) {
        return;
    }
    PFX_ConsoleService.logStringMessage("PFX: " + msg);
}
function _error( msg ) {
    PFX_ConsoleService.logStringMessage("PFX ERROR: " + msg);
}
function _warning( msg ) {
    PFX_ConsoleService.logStringMessage("PFX WARN: " + msg);
}

var PFX_PrefObserver;
function PFX_PrefObserver() {
    this.register();
}

PFX_PrefObserver.prototype = {

  register: function() {
      var prefService 
      = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService);
      this._branch = prefService.getBranch( "pronto." );
      this._branch.QueryInterface(Components.interfaces.nsIPrefBranchInternal);
      this._branch.addObserver("", this, false);
   },

  unregister: function() {
      if(!this._branch) return;
      this._branch.removeObserver("", this);
  },

  observe: function(aSubject, aTopic, aData) {
      if ( aTopic != "nsPref:changed" ) return;
      if ( ! Components ) return;
      
      switch (aData) {
      case "toast.snooze":
          PFX_handleSnoozeChange( this._branch.getBoolPref( "toast.snooze" ));
          break;
      case "debug.snooze":
          PFX_DEBUG_SNOOZE = this._branch.getBoolPref('debug.snooze');
      }
  }
}
function nsProntoFox() { 

    PFX_DEBUG_ENABLED = PFX_PrefBranch.getBoolPref("debug.main") ;
    
    PFX_PrefObserver = new PFX_PrefObserver();
    PFX_PrefBranch.setBoolPref( "toast.snooze", false );

    _initialize();

}

nsProntoFox.prototype = {
    clearToastHistory: function( ) {
        _toastHistory = new Array();
        _toastTitleHistory = new Array();
    },
    addToToastHistory: function(  site_name,
                                  toast_type_id,
                                  toast_type,
                                  title,
                                  default_url,
                                  compare_url,
                                  sub_url,
                                  num_merchants,
                                  low_price,
                                  high_price,
                                  html,
                                  mini_html  ) {
        
        var toast = new Toast( site_name,
                               toast_type_id,
                               toast_type,
                               title,
                               default_url,
                               compare_url,
                               sub_url,
                               num_merchants,
                               low_price,
                               high_price,
                               html,
                               mini_html );

        if ( _toastHistory.length >= MAX_TOAST_HISTORY_LENGTH ) {
            _toastHistory.pop();
            _toastTitleHistory.pop();
        }
        
        _toastHistory.unshift( toast );
        _toastTitleHistory.unshift( title );
    },
    getToastTitleHistory: function( count ) {
        count.value = _toastTitleHistory.length;
        return _toastTitleHistory;
    },
    getToast: function( index, count ) {
        
        var values = new Array();

        if (( _toastHistory.length > 0 )
            && ( index >= 0 )
            && ( index < _toastHistory.length )) {

            values = _toastHistory[index].toArray();
        }
        
        count.value = values.length;
        return values;

    },

    getNextSessionID: function( ) {
        var session_id = _sessionCounter;
        _sessionCounter += 1;
        return session_id;
    },

    hasAlertService: function( ) {
        return _has_alert_service;
    },
    QueryInterface: function(iid) {
        if (!iid.equals(Components.interfaces.nsISupports) &&
            !iid.equals(PRONTOFOX_IID))
        throw Components.results.NS_ERROR_NO_INTERFACE;
        return this;
    }


};
function _initialize() {
    try {
        var alerts = Components.classes["@mozilla.org/alerts-service;1"].getService(Components.interfaces.nsIAlertsService);
        this._has_alert_service = true;
    } 
    catch (e) {
        this._has_alert_service = false;
    }
 
}
function Toast( site_name,
                toast_type_id,
                toast_type,
                title,
                default_url,
                compare_url,
                sub_url,
                num_merchants,
                low_price,
                high_price,
                html,
                mini_html ) {

    this._init( site_name,
                toast_type_id,
                toast_type,
                title,
                default_url,
                compare_url,
                sub_url,
                num_merchants,
                low_price,
                high_price,
                html,
                mini_html );
}

Toast.prototype = {
    _init: function ( site_name,
                      toast_type_id,
                      toast_type,
                      title,
                      default_url,
                      compare_url,
                      sub_url,
                      num_merchants,
                      low_price,
                      high_price,
                      html,
                      mini_html ) {

        this.fields = new Array;
        this.fields.push( site_name );
        this.fields.push( toast_type_id );
        this.fields.push( toast_type );
        this.fields.push( title );
        this.fields.push( default_url );
        this.fields.push( compare_url );
        this.fields.push( sub_url );
        this.fields.push( num_merchants );
        this.fields.push( low_price );
        this.fields.push( high_price );
        this.fields.push( html );
        this.fields.push( mini_html );
    },
    
    toArray: function() 
    {
        return this.fields;
    }
}

var PFX_snoozeTimer = Components.classes["@mozilla.org/timer;1"].getService(Components.interfaces.nsITimer);
var PFX_SnoozeTimerObserver = {
    notify: function() {
        if ( PFX_DEBUG_ENABLED )
            PFX_DEBUG( "Snooze timer triggered." );
        
        PFX_PrefBranch.setBoolPref( "toast.snooze", false );
    }
}
function PFX_startSnoozeTimer() {

    var duration = 60 * 60 * 1000
            * PFX_PrefBranch.getIntPref( "toast.snooze.duration.hrs" );

    if ( PFX_DEBUG_SNOOZE )
        duration = PFX_PrefBranch.getIntPref( "toast.snooze.duration.hrs" );

    if ( PFX_DEBUG_ENABLED )
        PFX_DEBUG( "Snoozing for " + duration + " milliseconds" );
    
    PFX_snoozeTimer.initWithCallback
            ( PFX_SnoozeTimerObserver,
              duration,
              Components.interfaces.nsITimer.TYPE_ONE_SHOT );

}
function PFX_handleSnoozeChange( should_snooze )
{
    if ( PFX_DEBUG_ENABLED )
        PFX_DEBUG( "Snooze state changed to: " + should_snooze );
    
    PFX_snoozeTimer.cancel();
    
    if ( ! should_snooze ) 
        return;

    PFX_startSnoozeTimer();

}

var nsProntoFoxModule = {

    registerSelf: function(compMgr, fileSpec, location, type) {
        compMgr = compMgr.QueryInterface(Components.interfaces.nsIComponentRegistrar);
        compMgr.registerFactoryLocation(PRONTOFOX_CID,
                                        "ProntoFox XPCOM",
                                        PRONTOFOX_CONTRACTID,
                                        fileSpec,
                                        location,
                                        type);
    },
    
    getClassObject: function(compMgr, cid, iid) {
        if (!cid.equals(PRONTOFOX_CID))
            throw Components.results.NS_ERROR_NO_INTERFACE;
        if (!iid.equals(Components.interfaces.nsIFactory))
            throw Components.results.NS_ERROR_NOT_IMPLEMENTED;
        return nsProntoFoxFactory;   
    },
    canUnload: function(compMgr) { return true; }
};

var nsProntoFoxFactory = {
    createInstance: function(outer, iid) {
        if (outer != null)
            throw Components.results.NS_ERROR_NO_AGGREGATION;
        if (!iid.equals(PRONTOFOX_IID) &&
            !iid.equals(Components.interfaces.nsISupports))
            throw Components.results.NS_ERROR_INVALID_ARG;
        return new nsProntoFox();
    }
};

function NSGetModule(comMgr, fileSpec) { return nsProntoFoxModule; }

